local _classMap = {}--#map<#string,#Class>
local _extensionsMap = {}--#map<#string,#list<#function>>
local _assemblerList = {}--#list<#function>


---
--@module Addon
local addon = {} --#Addon
addon.name = "ActionDurationReminder" --#string
addon.version = "3.0.22" --#string
---
--@field [parent=#global] #Addon ActionDurationReminder
ActionDurationReminder = addon

---
--@param #string className
--@param #function constructor
--@param #function startFunc
--@return #Class class
function addon.AddClass(className, constructor, assembleFunc)
    ---
    -- @type Class
    local class = {
        className = className,
    }
    local o = class
    o.__index = o
    ---
    --@param #Class self
    --@param #string extensionName
    class.CallExtension=function(self, extensionName, ...)
        addon.CallExtension(className..':'..extensionName,self,...)
    end
    setmetatable(class, {
        __call = function (cls, ...)
            local instance = setmetatable({}, cls)
            if constructor then constructor(instance, ...) end
            return instance
        end,
    })
    _classMap[className] = class
    if assembleFunc then table.insert(_assemblerList, function() assembleFunc(class) end) end
    return class
end

---
--@param #string fullExtensionName
--@param #function extensionFunc
function addon.AddExtension(fullExtensionName, extensionFunc)
    local extensionFuncList = _extensionsMap[fullExtensionName] --#list<#function>
    if not extensionFuncList then
        extensionFuncList = {}
        _extensionsMap[fullExtensionName] = extensionFuncList
    end
    table.insert(extensionFuncList,extensionFunc)
end

---
--@param #string fullExtensionName
function addon.CallExtension(fullExtensionName, ...)
    local extensionFuncList = _extensionsMap[fullExtensionName]
    if extensionFuncList then
        for i, extensionFunc in ipairs(extensionFuncList) do
            extensionFunc(...)
        end
    end
end

---
--@param #string className
--@return #Class class
function addon.GetClass(className)
    return _classMap[className]
end

---
--@return #table
function addon.SavedVars()
    return nil -- this will be overriden
end

local function Assemble()
    for i, assembler in ipairs(_assemblerList) do
        assembler()
    end
end

local function OnAddOnLoaded(eventCode, addonName)
    if addon.name ~= addonName then return end
    EVENT_MANAGER:UnregisterForEvent(addonName, eventCode)
    Assemble()
    addon.CallExtension("addon:Start", addon)
    addon.CallExtension("addon:OnStarted", addon)
end

--========================================
EVENT_MANAGER:RegisterForEvent(addon.name, EVENT_ADD_ON_LOADED, OnAddOnLoaded)

return addon
