local addon = ActionDurationReminder --Addon#Addon

---
--@type SettingsSavedVars
local settingsSavedVarsDefaults = {
    settingsAccountWide= true,
}

---
--@module Settings
--@extends Addon#Class
local module = addon.AddClass("adr.Settings",
    ---
    -- @callof #Settings
    -- @param #Settings self
    -- @return #Settings
    function(self)
        self.name = "ADRSV"
        self.version = "1.0"
        self.savedVars = nil --#SettingsSavedVars
        self:Build()
    end
    ,
    ---
    --@param #Settings class
    function(class)
        addon.AddExtension("addon:Start",function() class() end)
    end
)--#Settings


---
--@param #Settings self
function module:Build()
    addon.SavedVars = function() return self.savedVars end
    local defaults={}
    for key, var in pairs(settingsSavedVarsDefaults) do
    	defaults[key] = var
    end
    self:CallExtension("BuildDefaults", defaults)
    local accountSavedVars = ZO_SavedVars:NewAccountWide( self.name, self.version, nil, defaults)--#SettingsSavedVars
    local characterSavedVars = ZO_SavedVars:New( self.name, self.version, nil, defaults)--#SettingsSavedVars
    self.savedVars = characterSavedVars.settingsAccountWide and accountSavedVars or characterSavedVars

    --- Init Menu
    local LAM2 = LibStub("LibAddonMenu-2.0")
    if LAM2 == nil then return end
    local panelData = {
        type = 'panel',
        name = addon.name,
        displayName = "ADR Settings",
        author = "Cloudor",
        version = addon.version,
        website = "http://www.esoui.com/downloads/info1536-ActionDurationReminder.html",
        slashCommand = "/adrset",
        registerForRefresh = true,
        registerForDefaults = true,
    }
    LAM2:RegisterAddonPanel('ADRAddonOptions', panelData)
    local gettext = LibStub("LibTextDict")("ActionDurationReminder").gettext
    local optionsData = {}
    optionsData[#optionsData+1] = {
        type = "checkbox",
        name = gettext("Account Wide Configuration"),
        getFunc = function() return characterSavedVars.settingsAccountWide end,
        setFunc = function(value)
            characterSavedVars.settingsAccountWide = value
            self.savedVars = value and accountSavedVars or characterSavedVars
        end,
        width = "full",
        default = true,
    }
    self:CallExtension("BuildOptionsData",optionsData)
    LAM2:RegisterOptionControls('ADRAddonOptions', optionsData)
end

--========================================
return module
