local addon = ActionDurationReminder --Addon#Addon

local SPECIAL_ABILITY_IDS = {
    TAUNT = 38541,
}

---
--@module Action
--@extends Addon#Class
local module = addon.AddClass(
    "adr.model.Action"
    ,
    ---
    -- @callof #Action
    -- @param #Action self
    -- @param #number slotNum
    -- @param #number pairIndex
    -- @param #boolean pairUltimate
    -- @return #Action
    function(self,slotNum, weaponPairIndex, weaponPairUltimate)
        self.slotNum = slotNum --#number
        local Ability = addon.GetClass("adr.model.Ability") --adr.model.Ability#Ability
        self.ability = Ability(GetSlotBoundId(slotNum),GetSlotName(slotNum),GetSlotTexture(slotNum)) --adr.model.Ability#Ability
        local channeled,castTime,channelTime = GetAbilityCastInfo(self.ability.id)
        self.castTime = castTime or 0 --#number
        self.startTime = GetGameTimeMilliseconds() --#number
        self.duration = GetAbilityDuration(self.ability.id) --#number
        self.endTime = self.duration==0 and 0 or self.startTime + self.duration--#number
        self.overideAction = nil --#Action

        self.weaponPairIndex = weaponPairIndex --#number
        self.weaponPairUltimate = weaponPairUltimate --#boolean
        local target = GetAbilityTargetDescription(self.ability.id)
        local isGround = target==GetString(SI_ABILITY_TOOLTIP_TARGET_TYPE_GROUND)
        local isArea = target==GetString(SI_ABILITY_TOOLTIP_TARGET_TYPE_AREA)
        local isSelf = target== GetString(SI_ABILITY_TOOLTIP_RANGE_SELF)
        local isTank = GetAbilityRoles(self.ability.id)
        ---
        --@type Flags
        self.flags = {
            isArea = isArea,
            isGround = isGround,
            isSelf = isSelf,
            isTank = isTank,
            shifted = false,
        }--#Flags
        self.data = {} -- #table to store data in
        self.effectMap = {} -- #map<#number,adr.model.ActionEffect#ActionEffect>
        self.stackCount = 0
    end
) --#Action


---
--@param #Action self
--@return #number duration
function module:Duration()
    local optEffect = self:OptEffect()
    return optEffect and optEffect.duration or self.duration
end

---
--@param #Action self
--@return #number endTime
function module:EndTime()
    local optEffect = self:OptEffect()
    return optEffect and optEffect.endTime or (self.endTime>0 and self.endTime or self.startTime)
end

---
--@pram #Action self
--@return #boolean whether on player
function module:IsOnPlayer()
    if self.flags.isSelf then return true end
    for key, var in pairs(self.effectMap) do
        local effect = var --adr.model.Effect#Effect
        if effect:IsOnPlayer() then return true end
    end
    return false
end

---
--@param #Action self
--@param adr.model.Effect#Effect effect
--@return #boolean whether match
function module:MatchNewEffect(effect)
    -- 0. filter ended action
    if self.endTime>self.startTime and self.endTime+300<effect.startTime then
        return false
    end
    -- 1. taunt
    if effect.ability.id==SPECIAL_ABILITY_IDS.TAUNT and self.flags.isTank then
        return true
    end
    -- 2. fast check already matched effects
    local strict = effect.startTime>self.startTime + 2000
    for key, var in pairs(self.effectMap) do
        local e = var --adr.model.Effect#Effect
        if effect.ability.id == e.ability.id then return true end
        if effect.startTime>e.startTime then strict = true end
    end
    -- 3. check ability match
    return self.ability:Match(effect.ability, strict)
end

---
--@param #Action self
--@param adr.model.Effect#Effect effect
--@return #boolean whether match
function module:MatchOldEffect(effect)
    -- 1. taunt
    if effect.ability.id==SPECIAL_ABILITY_IDS.TAUNT and self.flags.isTank then
        return true
    end
    -- 2. fast check already matched effects
    for key, var in pairs(self.effectMap) do
        local e = var --adr.model.Effect#Effect
        if e.ability.id == effect.ability.id and (e.unitId == effect.unitId or effect.unitId==0) then
            return true
        end
    end
    return false
end

---
--@param #Action self
--@return adr.model.Effect#Effect
function module:OptEffect()
    local optEffect = nil --adr.model.Effect#Effect
    for key, var in pairs(self.effectMap) do
        local effect = var --adr.model.Effect#Effect
        if not optEffect or optEffect.endTime < effect.endTime then
            optEffect = effect
        end
    end
    return optEffect
end

---
--@param #Action self
--@param adr.model.Effect#Effect effect
function module:PurgeEffect(effect)
    self.effectMap[effect.ability.id] = nil
    local hasOtherEffect = false
    for key, var in pairs(self.effectMap) do
        hasOtherEffect = true
    end
    if not hasOtherEffect then self.endTime = GetGameTimeMilliseconds() end
end

---
--@param #Action self
--@param adr.model.Effect#Effect effect
function module:SaveEffect(effect)
    self.effectMap[effect.ability.id] = effect
end

---
--@param #Action self
--@return #number startTime
function module:StartTime()
    local optEffect = self:OptEffect()
    return optEffect and optEffect.startTime or self.startTime
end
--========================================
return module
