local addon = ActionDurationReminder --Addon#Addon
local bar --#Bar

---
--@type BarSavedVars
local barSavedVarsDefaults = {
    barShowShift = true,
    barShiftOffsetX = 0,
    barShiftOffsetY = 5,
    barLabelFontName = "BOLD_FONT",
    barLabelFontSize = 18,
    barLabelYOffset = 0,
    barLabelIgnoreDecimal = true,
    barLabelIgnoreDeciamlThreshold = 10,
    barCooldownVisible = true,
    barCooldownColor = {1,1,0},
    barCooldownOpacity = 100,
    barCooldownThickness = 2,
}

---
--@return #BarSavedVars
local function SavedVars()
    return addon.SavedVars()
end

local shiftBarFrame = nil







---
--@module Bar
--@extends Addon#Class
local module = addon.AddClass(
    "adr.ui.Bar"
    ,
    ---
    -- @callof #Bar
    -- @param #Bar self
    -- @return #Bar
    function(self)
        self.mainBarWidgets = {}--#map<#number, adr.ui.Widget#Widget>
        self.shiftBarWidgets = {}--#map<#number, adr.ui.Widget#Widget>
        self.appendBarWidgets = {}--#map<#number, adr.ui.Widget#Widget>
        addon.AddExtension("adr.Core:Update",function(core)
            self:OnCoreUpdate(core)
        end)
    end
    ,
    ---
    --@param #Bar class
    function(class)
        addon.AddExtension("adr.Settings:BuildDefaults", function(settings, defaults)
            for key, var in pairs(barSavedVarsDefaults) do
                defaults[key] = var
            end
        end)
        addon.AddExtension("adr.Settings:BuildOptionsData", function(settings, optionsData)
            local Widget = addon.GetClass("adr.ui.Widget") --adr.ui.Widget#Widget
            local gettext = LibStub("LibTextDict")("ActionDurationReminder").gettext
            optionsData[#optionsData+1] = {
                type = "checkbox",
                name = gettext("Shift Bar Enabled"),
                getFunc = function() return SavedVars().barShowShift end,
                setFunc = function(value) SavedVars().barShowShift = value end,
                width = "full",
                default = barSavedVarsDefaults.barShowShift,
            }
            optionsData[#optionsData+1] = {
                type = "button",
                name = gettext("Move Shift Bar"),
                func = function()
                    SCENE_MANAGER:Hide("gameMenuInGame")
                    bar:OpenShiftBarFrame()
                    zo_callLater(function()
                        SetGameCameraUIMode(true)
                    end, 10)
                end,
                width = "full",
                disabled = function() return not SavedVars().barShowShift end,
            }
            optionsData[#optionsData+1] = {
                type = "dropdown",
                name = gettext("Label Font Name"),
                choices = {"MEDIUM_FONT", "BOLD_FONT", "CHAT_FONT", "ANTIQUE_FONT", "HANDWRITTEN_FONT", "STONE_TABLET_FONT", "GAMEPAD_MEDIUM_FONT", "GAMEPAD_BOLD_FONT"},
                getFunc = function() return SavedVars().barLabelFontName end,
                setFunc = function(value) SavedVars().barLabelFontName = value; bar:UpdateWidgets(Widget.UpdateFont) end,
                width = "full",
                default = barSavedVarsDefaults.barLabelFontName,
            }
            optionsData[#optionsData+1] = {
                type = "slider",
                name = gettext("Label Font Size"),
                --tooltip = "",
                min = 12, max = 24, step = 1,
                getFunc = function() return SavedVars().barLabelFontSize end,
                setFunc = function(value) SavedVars().barLabelFontSize = value ; bar:UpdateWidgets(Widget.UpdateFont) end,
                width = "full",
                default = barSavedVarsDefaults.barLabelFontSize,
            }
            optionsData[#optionsData+1] = {
                type = "slider",
                name = gettext("Label Vertical Offset"),
                min = -50, max = 50, step = 5,
                getFunc = function() return SavedVars().barLabelYOffset end,
                setFunc = function(value) SavedVars().barLabelYOffset = value ; bar:UpdateWidgets(Widget.UpdateLabelYOffset) end,
                width = "full",
                default = barSavedVarsDefaults.barLabelYOffset,
            }
            optionsData[#optionsData+1] = {
                type = "checkbox",
                name = gettext("Label Ignore Decimal Part"),
                getFunc = function() return SavedVars().barLabelIgnoreDecimal end,
                setFunc = function(value) SavedVars().barLabelIgnoreDecimal = value end,
                width = "full",
                default = barSavedVarsDefaults.barLabelIgnoreDecimal,
            }
            optionsData[#optionsData+1] = {
                type = "slider",
                name = gettext("Label Ignore Decimal Part Threshold"),
                min = 0, max = 30, step = 0.5,
                getFunc = function() return SavedVars().barLabelIgnoreDeciamlThreshold end,
                setFunc = function(value) SavedVars().barLabelIgnoreDeciamlThreshold = value end,
                width = "full",
                disabled = function() return not SavedVars().barLabelIgnoreDecimal end,
                default = barSavedVarsDefaults.barLabelIgnoreDeciamlThreshold,
            }
            optionsData[#optionsData+1] = {
                type = "checkbox",
                name = gettext("Line Enabled"),
                getFunc = function() return SavedVars().barCooldownVisible end,
                setFunc = function(value) SavedVars().barCooldownVisible = value; bar:UpdateWidgets(Widget.UpdateCooldown) end,
                width = "full",
                default = barSavedVarsDefaults.barCooldownVisible,
            }
            optionsData[#optionsData+1] = {
                type = "slider",
                name = gettext("Line Thickness"),
                min = 1, max = 5, step = 1,
                getFunc = function() return SavedVars().barCooldownThickness end,
                setFunc = function(value) SavedVars().barCooldownThickness = value ; bar:UpdateWidgets(Widget.UpdateCooldown) end,
                disabled = function() return not SavedVars().barCooldownVisible end,
                width = "full",
                default = barSavedVarsDefaults.barCooldownThickness,
            }
            optionsData[#optionsData+1] = {
                type = "slider",
                name = gettext("Line Opacity %"),
                min = 10, max = 100, step = 10,
                getFunc = function() return SavedVars().barCooldownOpacity end,
                setFunc = function(value) SavedVars().barCooldownOpacity = value ; bar:UpdateWidgets(Widget.UpdateCooldown) end,
                disabled = function() return not SavedVars().barCooldownVisible end,
                width = "full",
                default = barSavedVarsDefaults.barCooldownOpacity,

            }
            optionsData[#optionsData+1] = {
                type = "colorpicker",
                name = gettext("Line Color"),
                getFunc = function() return unpack(SavedVars().barCooldownColor) end,
                setFunc = function(r,g,b,a) SavedVars().barCooldownColor={r,g,b}; bar:UpdateWidgets(Widget.UpdateCooldown) end,
                width = "full",
                disabled = function() return not SavedVars().barCooldownVisible end,
                default = {
                    r = barSavedVarsDefaults.barCooldownColor[1],
                    g = barSavedVarsDefaults.barCooldownColor[2],
                    b = barSavedVarsDefaults.barCooldownColor[3],
                },
            }
        end)
        addon.AddExtension("addon:OnStarted",function(addon)
            bar=class()
        end)
    end
) --#Bar



---
--@param #Bar self
function module:OpenShiftBarFrame()
    local gettext = LibStub("LibTextDict")("ActionDurationReminder").gettext
    local slot3 = ZO_ActionBar_GetButton(3).slot
    local slot7 = ZO_ActionBar_GetButton(7).slot
    if not shiftBarFrame then
        shiftBarFrame = WINDOW_MANAGER:CreateControl(nil, slot3, CT_BACKDROP) --BackdropControl#BackdropControl
        local width = slot7:GetRight() - slot3:GetLeft()
        local _,height = slot7:GetDimensions()
        shiftBarFrame:SetDimensions(width, height)
        shiftBarFrame:SetMouseEnabled(true)
        shiftBarFrame:SetMovable(true)
        shiftBarFrame:SetDrawLayer(DL_COUNT)
        shiftBarFrame:SetCenterColor(0,0,1,0.5)
        shiftBarFrame:SetEdgeTexture('',1,1,1,0)
        shiftBarFrame:SetHandler('OnMoveStop', function()
            local left = shiftBarFrame:GetLeft()
            local bottom = shiftBarFrame:GetBottom()
            SavedVars().barShiftOffsetX = left - slot3:GetLeft()
            SavedVars().barShiftOffsetY = bottom - slot3:GetTop()
            local Widget = addon.GetClass("adr.ui.Widget") --adr.ui.Widget#Widget
            self:UpdateWidgets(Widget.UpdateShiftOffset)
            zo_callLater(function()
                SetGameCameraUIMode(true)
            end, 10)
        end)
        local label = WINDOW_MANAGER:CreateControl(nil, shiftBarFrame, CT_LABEL) --LabelControl#LabelControl
        label:SetFont('$(MEDIUM_FONT)|$(KB_18)|soft-shadow-thin')
        label:SetColor(1,1,1)
        label:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
        label:SetVerticalAlignment(TEXT_ALIGN_CENTER)
        label:SetAnchor(CENTER)
        label:SetDrawLayer(DL_COUNT)
        label:SetDrawLevel(1)
        label:SetText(gettext("ADR Shift Bar Frame"))
        local labelClose = WINDOW_MANAGER:CreateControl(nil, shiftBarFrame, CT_LABEL) --LabelControl#LabelControl
        labelClose:SetFont('$(MEDIUM_FONT)|$(KB_18)|soft-shadow-thin')
        labelClose:SetColor(1,1,1)
        labelClose:SetHorizontalAlignment(TEXT_ALIGN_RIGHT)
        labelClose:SetVerticalAlignment(TEXT_ALIGN_BOTTOM)
        labelClose:SetAnchor(BOTTOMRIGHT, nil, BOTTOMRIGHT, -5, -2)
        labelClose:SetDrawLayer(DL_COUNT)
        labelClose:SetDrawLevel(1)
        labelClose:SetText('[X]')
        labelClose:SetMouseEnabled(true)
        labelClose:SetHandler('OnMouseUp', function()
            shiftBarFrame:SetHidden(true)
        end)
    end
    shiftBarFrame:SetHidden(false)
    shiftBarFrame:ClearAnchors()
    shiftBarFrame:SetAnchor(BOTTOMLEFT, slot3, TOPLEFT, SavedVars().barShiftOffsetX, SavedVars().barShiftOffsetY)
end

---
--@param #Bar self
--@param adr.Core#Core core
function module:OnCoreUpdate(core)
    local now = GetGameTimeMilliseconds()
    local showedActionMap = {} --#map<#number,adr.model.Action#Action>
    local Widget = addon.GetClass("adr.ui.Widget")--adr.ui.Widget#Widget
    -- 1. show in action bar widgets
    for slotNum = 3,8 do
        local widget = self.mainBarWidgets[slotNum]
        local abilityId = GetSlotBoundId(slotNum) --#number
        local action = core.idActionMap[abilityId]
        if action then
            action.flags.shifted = false
            showedActionMap[action.ability.id] = action
            if not widget then
                widget = Widget(slotNum, false)
                self.mainBarWidgets[slotNum] = widget
            end
            widget:UpdateUI(action, now)
        elseif widget then
            widget:Hide()
        end
    end
    -- 2. clean shift and extend widgets
    for key,var in pairs(self.shiftBarWidgets) do
        local widget = var --adr.ui.Widget#Widget
        widget:Hide()
    end
    for key,var in pairs(self.appendBarWidgets) do
        local widget = var --adr.ui.Widget#Widget
        widget:Hide()
    end
    if not SavedVars().barShowShift then return end
    -- 3. prepare to show shift and extend
    local toShowActionMap = {} --#map<#number,adr.model.Action#Action>
    local toShowIdList = {} --#list<#number>
    for key, var in pairs(core.idActionMap) do
        if not showedActionMap[key] then
            local action = var --adr.model.Action#Action
            action.flags.shifted = true
            toShowActionMap[key] = var
            toShowIdList[#toShowIdList+1] = key
        end
    end
    if #toShowIdList==0 then return end
    -- 3.$ sort later actions show first
    table.sort(toShowIdList, function(id1,id2)return toShowActionMap[id1]:StartTime() > toShowActionMap[id2]:StartTime() end)
    local appendIndex = 0
    local cooldownThickness = SavedVars().barCooldownThickness
    for i=1,#toShowIdList do
        local id = toShowIdList[i]
        local action = toShowActionMap[id]
        local slotNum = action.slotNum
        local inAppend = core.weaponPairInfo.ultimate and ( action.weaponPairIndex ~= core.weaponPairInfo.index) or action.weaponPairUltimate
        local widget = self.shiftBarWidgets[slotNum]
        if inAppend or (widget and widget.visible) then
            appendIndex=appendIndex+1
            widget = self.appendBarWidgets[appendIndex]
            if not widget then
                widget = Widget(8,true,appendIndex)
                self.appendBarWidgets[appendIndex] = widget
            end
        elseif not widget then
            widget = Widget(slotNum,true)
            self.shiftBarWidgets[slotNum] = widget
        end
        widget:UpdateUI(action, now)
        if now > action:EndTime() then
            widget.backdrop:SetDimensions(50,50)
        else
            widget.backdrop:SetDimensions(50-math.max(0,cooldownThickness-2),50-math.max(0,cooldownThickness-2))
        end
    end
end

---
--@param #Bar self
--@param #function func
function module:UpdateWidgets(func)
    for key, var in pairs(self.mainBarWidgets) do
        func(var)
    end
    for key, var in pairs(self.shiftBarWidgets) do
        func(var)
    end
    for key, var in pairs(self.appendBarWidgets) do
        func(var)
    end
end

--========================================
return module
