local addon = ActionDurationReminder -- Addon#Addon

---
--@return adr.ui.Bar#BarSavedVars
local function SavedVars()
    return addon.SavedVars()
end
---
--@return #string font
local function GetLabelFont()
    return "$("..SavedVars().barLabelFontName..")|"..SavedVars().barLabelFontSize.."|thick-outline"
end
---
--@module Widget
--@extends Addon#Class
local module = addon.AddClass(
    "adr.ui.Widget"
    ,
    ---
    -- @callof #Widget
    -- @param #Widget self
    -- @param adr.ui.Bar#Bar bar
    -- @param #number slotNum
    -- @param #boolean shift
    -- @param #number appendIndex
    -- @return #Widget
    function(self, slotNum, shift, appendIndex)
        self.slotNum = slotNum --#number
        self.shift = shift --#boolean
        self.appendIndex = appendIndex --#number
        --
        self.visible = true
        local slot = ZO_ActionBar_GetButton(slotNum).slot --Control#Control
        local slotIcon = slot:GetNamedChild("Icon")
        local flipCard = slot:GetNamedChild("FlipCard")
        self.slotIcon = slotIcon --Control#Control
        self.flipCard = flipCard --Control#Control
        --========================================
        local backdrop = nil
        local background = nil
        local barCooldownThickness = SavedVars().barCooldownThickness
        if shift then
            local offsetX = SavedVars().barShiftOffsetX
            local offsetY = SavedVars().barShiftOffsetY
            backdrop = WINDOW_MANAGER:CreateControl(nil, slot, CT_TEXTURE)
            self.backdrop = backdrop --TextureControl#TextureControl
            backdrop:SetDrawLayer(DL_BACKGROUND)
            if appendIndex then
                backdrop:SetAnchor(BOTTOM, slot, TOP, offsetX + (appendIndex-1) * 55, offsetY)
            else
                backdrop:SetAnchor(BOTTOM, slot, TOP,offsetX , offsetY)
            end
            backdrop:SetDimensions(50-math.max(0,barCooldownThickness-2),50-math.max(0,barCooldownThickness-2))
            backdrop:SetTexture("esoui/art/actionbar/abilityframe64_up.dds")
            backdrop:SetTextureCoords(0, .625, 0, .8125)
            background = WINDOW_MANAGER:CreateControl(nil, backdrop, CT_TEXTURE)
            self.background = background--TextureControl#TextureControl
            background:SetDrawLayer(DL_CONTROLS)
            background:SetAnchor(TOPLEFT, backdrop, TOPLEFT, 2, 2)
            background:SetAnchor(BOTTOMRIGHT, backdrop, BOTTOMRIGHT, -2, -2 )
        end
        local label = WINDOW_MANAGER:CreateControl(nil, backdrop or slotIcon, CT_LABEL)
        self.label = label --LabelControl#LabelControl
        label:SetFont(GetLabelFont())
        label:SetColor(1,1,1)
        label:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
        label:SetVerticalAlignment(TEXT_ALIGN_BOTTOM)
        label:SetWrapMode(TEXT_WRAP_MODE_ELLIPSIS)
        label:SetAnchor(BOTTOM, backdrop or slotIcon, BOTTOM, 0, SavedVars().barLabelYOffset + 5)
        label:SetDrawLayer(DL_TEXT)
        local countLabel = WINDOW_MANAGER:CreateControl(nil, backdrop or slotIcon, CT_LABEL)
        self.countLabel = countLabel --LabelControl#LabelControl
        countLabel:SetFont(GetLabelFont())
        countLabel:SetColor(1,1,1)
        countLabel:SetHorizontalAlignment(TEXT_ALIGN_CENTER)
        countLabel:SetVerticalAlignment(TEXT_ALIGN_TOP)
        countLabel:SetWrapMode(TEXT_WRAP_MODE_ELLIPSIS)
        countLabel:SetAnchor(TOPRIGHT, backdrop or slotIcon, TOPRIGHT, 0, - SavedVars().barLabelYOffset - 5)
        countLabel:SetDrawLayer(DL_TEXT)
        local cooldown = WINDOW_MANAGER:CreateControl(nil, backdrop or slot, CT_COOLDOWN)
        self.cooldown = cooldown --CooldownControl#CooldownControl
        cooldown:SetDrawLayer(DL_BACKGROUND)
        if background then
            cooldown:SetAnchor( TOPLEFT, background, TOPLEFT, -barCooldownThickness,-barCooldownThickness )
            cooldown:SetAnchor( BOTTOMRIGHT, background, BOTTOMRIGHT,barCooldownThickness, barCooldownThickness )
        else
            cooldown:SetAnchor( CENTER, flipCard, CENTER)
            local slotWidth,slotHeight = slot:GetDimensions()
            cooldown:SetDimensions( slotWidth, slotHeight)
        end
        cooldown:SetFillColor(unpack(SavedVars().barCooldownColor))
        if SavedVars().barCooldownOpacity < 100 then cooldown:SetAlpha(SavedVars().barCooldownOpacity/100) end
        self.cdMark = 0
    end
    ,
    ---
    --@param #Widget class
    function(class)
    -- Start Logic
    end
) --#Widget

---
--@param #Widget self
function module:Hide()
    if self.backdrop then self.backdrop:SetHidden(true) end
    if self.background then self.background:SetHidden(true) end
    self.label:SetHidden(true)
    self.countLabel:SetHidden(true)
    self.cooldown:SetHidden(true)
    self.visible = false
    local _,_,_,_,flipOffset =  self.flipCard:GetAnchor(0)
    if flipOffset>0 then
        local flipParent = self.flipCard:GetParent()
        self.flipCard:ClearAnchors()
        self.flipCard:SetAnchor(TOPLEFT, flipParent, TOPLEFT, 0, 0)
        self.flipCard:SetAnchor(BOTTOMRIGHT, flipParent, BOTTOMRIGHT, 0, 0)
    end
end

---
--@param #Widget self
--@param adr.model.Action#Action action
--@param #number now
function module:UpdateUI(action, now)
    self.visible = true
    if self.backdrop then self.backdrop:SetHidden(false) end
    if self.background then
        self.background:SetTexture(action.ability.icon)
        self.background:SetHidden(false)
    end
    local endTime = action:EndTime()
    local remain = math.max(endTime-now,0)
    local hint = string.format('%.1f', remain/1000)
    if SavedVars().barLabelIgnoreDecimal and remain/1000>SavedVars().barLabelIgnoreDeciamlThreshold then
        hint = string.format('%d', remain/1000)
    end
    self.label:SetText(hint)
    self.label:SetHidden(false)
    if action.stackCount ~= nil and action.stackCount > 0 then
        self.countLabel:SetText(action.stackCount)
        self.countLabel:SetHidden(false)
    else
        self.countLabel:SetHidden(true)
    end
    local cdMark = endTime
    if remain > 7000 and action:Duration() > 8000 then
        cdMark = action.endTime - 7000
        if self.cdMark ~= cdMark then
            self.cdMark = cdMark
            local scale = action.duration/1000 - 7
            local scaledTotal = action.duration * scale
            local scaledRemain = remain * scale
            self.cooldown:StartCooldown(scaledRemain, scaledTotal, CD_TYPE_RADIAL, CD_TIME_TYPE_TIME_UNTIL, false )
        end
    elseif remain > 0 then
        if self.cdMark ~= cdMark then
            self.cdMark = cdMark
            self.cooldown:StartCooldown(remain, 8000, CD_TYPE_RADIAL, CD_TIME_TYPE_TIME_UNTIL, false )
        end
    else
        self.cdMark = 0
        local numSemiSeconds = math.floor((now-action:EndTime())/200)
        if numSemiSeconds % 2 == 0 then
            --if action.inBar then widget.label:SetColor(0,1,0) else widget.label:SetColor(1,0,0) end
            self.label:SetHidden(true)
        end
    end
    local cdHidden = cdMark==0 or not SavedVars().barCooldownVisible
    self.cooldown:SetHidden(cdHidden)
end


---
--@param adr.ui.Widget#Widget self
function module:UpdateCooldown()
    if self.cooldown then
        self.cooldown:SetHidden(not SavedVars().barCooldownVisible)
        self.cooldown:SetFillColor(unpack(SavedVars().barCooldownColor))
        if SavedVars().barCooldownOpacity < 100 then
            self.cooldown:SetAlpha(SavedVars().barCooldownOpacity/100)
        else
            self.cooldown:SetAlpha(1)
        end
    end
    local barCooldownThickness = SavedVars().barCooldownThickness
    if self.backdrop then
        self.backdrop:SetDimensions(50-math.max(0,barCooldownThickness-2),50-math.max(0,barCooldownThickness-2))
    end
    if self.background and self.cooldown then
        self.cooldown:ClearAnchors()
        self.cooldown:SetAnchor( TOPLEFT, self.background, TOPLEFT, -barCooldownThickness,-barCooldownThickness )
        self.cooldown:SetAnchor( BOTTOMRIGHT, self.background, BOTTOMRIGHT,barCooldownThickness, barCooldownThickness )
    end
end


---
--@param adr.ui.Widget#Widget self
function module:UpdateFont()
    if self.label then self.label:SetFont(GetLabelFont()) end
    if self.countLabel then self.countLabel:SetFont(GetLabelFont()) end
end


---
--@param adr.ui.Widget#Widget self
function module:UpdateLabelYOffset()
    if self.label then
        self.label:ClearAnchors()
        self.label:SetAnchor(BOTTOM, self.label:GetParent(), BOTTOM, 0, SavedVars().barLabelYOffset + 5)
    end
    if self.countLabel then
        self.countLabel:ClearAnchors()
        self.countLabel:SetAnchor(TOPRIGHT, self.countLabel:GetParent(), TOPRIGHT, 0, - SavedVars().barLabelYOffset - 5)
    end
end

---
--@param adr.ui.Widget#Widget self
function module:UpdateShiftOffset()
    if not self.shift then return end
    local offsetX = SavedVars().barShiftOffsetX
    local offsetY = SavedVars().barShiftOffsetY
    if self.backdrop then
        local slot = self.backdrop:GetParent()
        self.backdrop:ClearAnchors()
        if self.appendIndex then
            self.backdrop:SetAnchor(BOTTOM, slot, TOP, offsetX + (self.appendIndex-1) * 55, offsetY)
        else
            self.backdrop:SetAnchor(BOTTOM, slot, TOP, offsetX , offsetY)
        end
    end
end

return module
